<?php
namespace Models;

// Модель строк чека
class Receiptlines extends \DB\SQL\Mapper {
	public $db, $f3;
	public $marking_group=array(
		1=>array('title'=>'Предметы одежды, белье постельное, столовое, туалетное и кухонное', 'pg'=>'lp'),
		2=>array('title'=>'Обувные товары', 'pg'=>'shoes'),
		3=>array('title'=>'Табачные изделия', 'pg'=>'tobacco'),
		4=>array('title'=>'Духи и туалетная вода', 'pg'=>'perfumery'),
		5=>array('title'=>'Шины и покрышки пневматические резиновые новые', 'pg'=>'tires'),
		6=>array('title'=>'Фотокамеры (кроме кинокамер), фотовспышки и лампы-вспышки', 'pg'=>'electronics'),
		7=>array('title'=>'Лекарственные препараты для медицинского применения', 'pg'=>'pharma'),
		8=>array('title'=>'Молочная продукция', 'pg'=>'milk'),
		9=>array('title'=>'Велосипеды и велосипедные рамы', 'pg'=>'bicycle'),
		10=>array('title'=>'Кресла-коляски', 'pg'=>'wheelchairs'),
		11=>array('title'=>'Меховые изделия', 'pg'=>'furs'),
		12=>array('title'=>'Альтернативная табачная продукция', 'pg'=>'otp'),
		13=>array('title'=>'Питьевая вода', 'pg'=>'water')
	);

	public function __construct($db, $f3) {
		parent::__construct($db, 'receiptlines' ); // подключаемся к таблице строк чека
		$this->db=$db;
		$this->f3=$f3;
	}

	// Выбираем строки из базы
	function select_lines($query, $option=null) {
		$products=new \DB\SQL\Mapper($this->db, 'products');
		$barcodes=new \DB\SQL\Mapper($this->db, 'barcodes');
		$lines=array();
		$parent=array();
		$modifiers=array();
		$sort_lines=array();
		$this->load($query, $option);
		if(!$this->dry()) {
			do {
				$l=$this->cast();
				$products->reset();
				$products->load(array('id=?', $l['product_id']), array('limit'=>1));
				if($products->marking_group>0) {
					$l['pg']=$this->marking_group[$products->marking_group]['pg'];
				}
				else {
					$l['pg']='';
				}
				$l['print_group']=$products->print_group;
				$l['egais']=$products->egais;
				$barcodes->reset();
				$barcodes->load(array('product_id=?', $products->id), array('limit'=>1));
				$l['barcode']=$barcodes->barcode;
				if($l['parent_id']>0) {
					$l['print_group']=$parent[$l['parent_id']]['print_group'];
					$modifiers[$l['parent_id']][]=$l;
				}
				else {
					$parent[$l['id']]=$l;
					$lines[]=$l;
				}
			}
			while($this->skip());
		}
		$this->reset();
		if($lines) {
			foreach ($lines as $l) {
				$sort_lines[]=$l;
				if(isset($modifiers[$l['id']])) {
					$modifiers[$l['id']][array_key_last($modifiers[$l['id']])]['last']=1;
					$sort_lines=array_merge($sort_lines, $modifiers[$l['id']]);
					unset($modifiers[$l['id']]);
				}
			}
		}
		if($modifiers) {
			foreach ($modifiers as $m) {
				$sort_lines=array_merge($sort_lines, $m);
			}
		}
		return $sort_lines;
	}

	// Добавляем строку в чек
	function add_line($receipt_id, $line, $count=1, $parent_id=0) {
		$line['parent_id']=$parent_id;
		if($parent_id>0) {
			$this->load(array('id=?', $parent_id), array('limit'=>1));
			if($this->dry()) {
				return array(
					'success'=>true,
					'txt'=>'not_found',
					'function'=>'$(".modifiers_false_'.$line['product_id'].'").show();'
				);
			}
			$line['kitchen']=$this->kitchen;
			$line['bar']=$this->bar;
		}
		$line['receipt_id']=$receipt_id;
		$count=str_replace(',', '.', $count);
		$line['count']=$count;
		$line['product_id']=$line['id'];
		$line['product_title']=$line['short_title'];
		if(!$line['product_title']) {
			$line['product_title']=$line['title'];
		}
		$line['price']=str_replace(',', '.', $line['price']);
		if(!isset($line['price0'])) {
			$line['price0']=$line['price'];
		}
		if(!isset($line['discount'])) {
			$line['discount']=0;
		}
		$line['loyalty']=0;
		unset($line['id']);
		$this->reset();
		if($parent_id>0) {
			$this->load(array('product_id=? AND parent_id=?', array($line['product_id'], $parent_id)), array('limit'=>1));
			if($count==0) {
				return $this->delete_line($this->id);
			}
		}
		if(!is_numeric($line['price']) || $count<=0) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_data',
				'function'=>'$(".modifiers_false_'.$line['product_id'].'").show();'
			);
		}
		$this->copyFrom($line);
		if($this->save()) {
			$id=$this->get('_id');
			if($parent_id>0) {
				$scrollto='#line_'.$parent_id;
				$id=$parent_id;
				//$this->db->exec("UPDATE receiptlines SET bar=0, kitchen=0 WHERE id=? OR parent_id=?", array($parent_id, $parent_id));
			}
			else {
				$scrollto='#line_'.$id;
			}
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'line'=>$line,
				'reload'=>array('.receiptlines', '#receipts_buttons', '.button_total'),
				'id'=>$id,
				'scroll'=>'.receiptlines_div',
				'scrollto'=>$scrollto,
				'close_all_window'=>true,
				'function'=>'$(".modifiers_success_'.$line['product_id'].'").show();'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error',
				'function'=>'$(".modifiers_false_'.$line['product_id'].'").show();'
			);
		}
	}

	// Меняем строку в чеке
	function change_line($line_id, $count, $price, $discount=0, $description=false, $loyalty=false) {
		$this->reset();
		$this->load(array('id=?', $line_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		if($this->parent_id==0 && ($this->kitchen>0 || $this->bar>0) && (isset($count) && $count!='')) {
			return array(
				'success'=>false,
				'txt'=>'position_is_already_in_progress'
			);
		}
		if($this->mark!='' && $count>1) {
			return array(
				'success'=>false,
				'txt'=>'quantity_marked_goods_may_not_exceed_1'
			);
		}
		$line=$this->cast();
		if($count) {
			$count=str_replace(',', '.', $count);
			$line['count']=$count;
		}
		if($price) {
			$price=str_replace(',', '.', $price);
			if($line['vat_type']>0) {
				$price-=$price/(100+$line['vat'])*$line['vat'];
			}
			$line['price']=$price;
		}
		$line['line_discount']=$discount;
		if(!$discount && $price) {
			$line['price0']=$line['price'];
		}
		if($description!==false) {
			$line['description']=$description;
		}
		if($loyalty!==false) {
			$line['loyalty']=$loyalty;
		}
		$this->copyFrom($line);
		if($this->update()) {
			if($line['parent_id']>0) {
				$val=array($line['parent_id'], $line['parent_id']);
			}
			else {
				$val=array($line['id'], $line['id']);
			}
			//$this->db->exec("UPDATE receiptlines SET bar=0, kitchen=0 WHERE id=? OR parent_id=?", $val);
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>array('.receiptlines', '#receipts_buttons', '.button_total'),
				'scroll'=>'.receiptlines_div',
				'scrollto'=>'#line_'.$line_id,
				'id'=>$line_id,
				'line'=>$line,
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Удаляем строку из чека
	function delete_line($line_id) {
		$this->reset();
		if($line_id=='all') {
			$res=$this->db->exec("DELETE FROM `receiptlines` WHERE `receipt_id`=?", $this->f3->get('SESSION.receipt_id'));
			if($res===false) {
				return array(
					'success'=>false,
					'txt'=>'db_error',
				);
			}
		}
		else {
			$this->load(array('id=?', $line_id), array('limit'=>1));
			if($this->dry()) {
				return array(
					'success'=>false,
					'txt'=>'not_found'
				);
			}
			$line=$this->cast();
			if($this->erase(array('id=? OR parent_id=?', array($line_id, $line_id)))===false) {
				return array(
					'success'=>false,
					'txt'=>'db_error',
					'function'=>'$(".modifiers_false_'.$line['product_id'].'").show();'
				);
			}
		}

		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>array('.receiptlines', '#receipts_buttons', '.button_total'),
			'line'=>$line,
			'close_all_window'=>true
		);
	}

	// Узнаем сумму строк
	function get_sum($lines, $discount=0) {
		$sum=0;
		$sum0=0;
		if($lines) {
			foreach ($lines as $l) {
				$s=($l['price']+($l['price']*(($l['vat']/100)*max(0, $l['vat_type']))));
				if($l['discount']) {
					$s-=$s*($l['discount']/100);
				}
				if($l['loyalty']>0) {
					$sum0+=$s*$l['count'];
				}
				else {
					$sum+=$s*$l['count'];
				}
			}
		}
		if($discount) {
			$sum-=$sum*($discount/100);
		}
		$sum+=$sum0;
		$sum=round($sum, 2);
		return $sum;
	}

	// Отмечаем, что позиция была напечатана на принтере кухни или бара
	function print_ok($line_id, $printer) {
		$this->reset();
		$this->load(array('id=?', $line_id), array('limit'=>1));
		if($this->dry()) {
			return false;
		}
		$data[$printer]=1;
		$this->copyFrom($data);
		if($this->update()===false) {
			return false;
		}
		return true;
	}

	// Сохраняем строки в базе данных
	function save_lines($receipt_id, $discount=0) {
		if(!$receipt_id) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}

		$res=$this->db->exec("UPDATE `receiptlines` SET `discount`=? WHERE `receipt_id`=? AND loyalty=0", array($discount, $receipt_id));
		if($res===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>array('.receiptlines', '#receipts_buttons'),
			'reload_menu'=>true,
			'close_all_window'=>true
		);
	}

	// Применение программ лояльности
	function apply_loyalty($receipt, $product, $parents, $loyalty) {
		$res=$this->db->exec("SELECT SUM(count) AS count FROM `receiptlines` WHERE receipt_id=? AND product_id=?", array($receipt['id'], $product['id']));
		$count=$res[0]['count']; // Общее кол-во товара
		// Выбираем все строки с товаром, и не измененной ценой
		$lines=$this->select_lines(array('receipt_id=? AND product_id=? AND loyalty=0', array($receipt['id'], $product['id'])), array('order'=>'count'));
		$selected=array(); // Выбранная программа
		// Если программ несколько, то перебираем программы лояльности, что бы узнать самую выгодную
		$new_price=false;
		foreach ($loyalty as $n => $l) {
			$items=json_decode($l['data'], true);
			// Находим параметры программы лояльности для товара
			$l_data=array();
			if(isset($items['product'][$product['id']])) { // Если параметры указаны для товара
				$l_data=$items['product'][$product['id']];
			}
			else { // Если параметры указаны для категории
				if($parents) { // Ищем параметры для категории
					foreach ($parents as $p) {
						if(isset($items['category'][$p['id']])) {
							$l_data=$items['category'][$p['id']];
						}
					}
				}
			}
			// Если параметры нашлись, вычисляем новую цену
			if($l_data) {
				if($l_data['quantity']=='')$l_data['quantity']=0;
				// Если кол-во товара соответствует необходимому
				if(($l_data['quantity_type']=='>' && $count>$l_data['quantity']) || ($l_data['quantity_type']=='=' && $count>=$l_data['quantity'])) {
					// Цена с учетом НДС, но без учета скидки
					$price=$product['price']+($product['price']/100*$product['vat'])*max(0, $product['vat_type']);
					// Вычисляем цену со скидкой, с учетом типа скидки
					if($l_data['discount_type']=='%') {
						$price-=($price/100*$l_data['discount']);
					}
					elseif($l_data['discount_type']=='='){
						$price=$l_data['discount'];
					}
					else {
						$price-=$l_data['discount'];
					}
					// Если нашли более выгодную программу
					if($new_price===false || $new_price>$price) {
						$new_price=$price;
						$selected=$l;
						$selected['data']=$l_data;
					}
				}
			}
		}

		// Применяем программу лояльности
		if($selected['data']['discount_type']=='%') {
			$discount=$selected['data']['discount'];
		}
		else {
			$discount=0;
		}

		$not_discount=$selected['data']['quantity']; // Кол-во к которому не надо применять скидку
		if($selected['data']['quantity_type']=='=') { // Если скидка на определенное кол-во
			$not_discount=$count%$not_discount;
		}

		// Если нужно применить не ко всему кол-ву
		if($not_discount>0) {
			// Ищем с троку у которой указано кол-во, к которому не нужно применять скидку, а вдруг такая есть
			$l_key=array_search($not_discount, array_column($lines, 'count'));
			if($l_key!==false) {
				// если такая строка нашлась, то удаляем ее, и применем скидку к оставшимся строкам
				unset($lines[$l_key]);
			}
			else {
				// Если такой строки нет, то надо работать с тем, что есть
				foreach ($lines as $n=>$l) {
					if($l['count']<=$not_discount) {
						$not_discount-=$l['count'];
						unset($lines[$n]);
					}
					else {
						// Если кол-во больше необходимого, то придется кол-во разделить на 2 строки
						// Оставляем в строке то кол-во, к которому будет применена скидку
						$lines[$n]['count']-=$not_discount;
						// Создаем новую строку с тем кол-вом, к которому не надо применять скидку
						$line=array(
							'id'=>$l['product_id'],
							'title'=>$l['product_title'],
							'price'=>$product['price'],
							'price0'=>$product['price'],
							'vat'=>$l['vat'],
							'vat_type'=>$l['vat_type']
						);
						$this->add_line($receipt['id'], $line, $not_discount);
					}
				}
			}
		}
		// Применяем скидку
		if($lines && $selected) {
			foreach ($lines as $l) {
				$this->change_line($l['id'], $l['count'], $new_price, $discount, 'Акция: '.$selected['title'], 1);
			}
		}
	}

	// Отмена акций
	function cancel_loyalty($receipt, $product) {
		$sql="UPDATE `receiptlines` SET `price`=?, `loyalty`=0, `line_discount`=0, `description`='' WHERE `receipt_id`=? AND `product_id`=? AND `loyalty`=1";
		$val=array(
			$product['price'],
			$receipt['id'],
			$product['id']
		);
		$this->db->exec($sql, $val);
	}

	function lines_bonus_pay($lines, $bonus_pay) {
		foreach ($lines as $l) {
			if($bonus_pay>0) {
				$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=(float) $price*((float) $l['discount']/100);
				$price-=$d_sum;
				if($l['line_discount']>0) {
					$price0=$l['price0']+($l['price0']*($l['vat']/100))*max(0, $l['vat_type']);
					$d_sum=(float) $price0*((float) $l['line_discount']/100);
				}
				if($bonus_pay>0) {
					$b=min($bonus_pay, $price);
					$price-=$b;
					$bonus_pay-=$b;
				}
				$this->db->exec("UPDATE receiptlines SET price=?, description=? WHERE id=?", array($price, 'Оплачено баллами: '.$b, $l['id']));
			}
			else {
				break;
			}
		}
		return; 
	}
}
